package de.szut.casino.blackjack;

import de.szut.casino.blackjack.dto.CreateBlackJackGameDto;
import de.szut.casino.user.UserEntity;
import de.szut.casino.user.UserService;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

@Slf4j
@RestController
public class BlackJackGameController {

    private final UserService userService;
    private final BlackJackService blackJackService;

    public BlackJackGameController(UserService userService, BlackJackService blackJackService) {
        this.blackJackService = blackJackService;
        this.userService = userService;
    }

    @PostMapping("/blackjack/{id}/hit")
    public ResponseEntity<Object> hit(@PathVariable Long id, @RequestHeader("Authorization") String token) {
        Optional<UserEntity> optionalUser = userService.getCurrentUser(token);

        if (optionalUser.isEmpty()) {
            return ResponseEntity.notFound().build();
        }

        UserEntity user = optionalUser.get();
        BlackJackGameEntity game = blackJackService.getBlackJackGame(id);
        if (game == null || !Objects.equals(game.getUserId(), user.getId())) {
            return ResponseEntity.notFound().build();
        }

        if (game.getState() != BlackJackState.IN_PROGRESS) {
            Map<String, String> errorResponse = new HashMap<>();
            errorResponse.put("error", "Invalid state");
            return ResponseEntity.badRequest().body(errorResponse);
        }

        return ResponseEntity.ok(blackJackService.hit(game));
    }

    @PostMapping("/blackjack/start")
    public ResponseEntity<Object> createBlackJackGame(@RequestBody @Valid CreateBlackJackGameDto createBlackJackGameDto, @RequestHeader("Authorization") String token) {
        Optional<UserEntity> optionalUser = userService.getCurrentUser(token);

        if (optionalUser.isEmpty()) {
            return ResponseEntity.notFound().build();
        }

        UserEntity user = optionalUser.get();
        BigDecimal balance = user.getBalance();
        BigDecimal betAmount = createBlackJackGameDto.getBetAmount();

        if (betAmount.compareTo(BigDecimal.ZERO) <= 0) {
            Map<String, String> errorResponse = new HashMap<>();
            errorResponse.put("error", "Invalid bet amount");
            return ResponseEntity.badRequest().body(errorResponse);
        }

        if (betAmount.compareTo(balance) > 0) {
            Map<String, String> errorResponse = new HashMap<>();
            errorResponse.put("error", "Insufficient funds");
            return ResponseEntity.badRequest().body(errorResponse);
        }

        return ResponseEntity.ok(blackJackService.createBlackJackGame(user, betAmount));
    }
}
