package de.szut.casino.blackjack;

import de.szut.casino.user.UserEntity;
import de.szut.casino.user.UserRepository;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;
import java.util.Random;

@Service
public class BlackJackService {
    private final BlackJackGameRepository blackJackGameRepository;
    private final UserRepository userRepository;

    public BlackJackService(BlackJackGameRepository blackJackGameRepository, UserRepository userRepository) {
        this.blackJackGameRepository = blackJackGameRepository;
        this.userRepository = userRepository;
    }

    private final Random random = new Random();

    public BlackJackGameEntity getBlackJackGame(Long id) {
        Optional<BlackJackGameEntity> optionalBlackJackGame = blackJackGameRepository.findById(id);
        return optionalBlackJackGame.orElse(null);
    }

    public BlackJackGameEntity createBlackJackGame(UserEntity user, BigDecimal betAmount) {
        BlackJackGameEntity game = new BlackJackGameEntity();
        game.setUser(user);
        game.setBet(betAmount);
        initializeDeck(game);

        for (int i = 0; i < 2; i++) {
            CardEntity playerCard = drawCardFromDeck(game);
            playerCard.setCardType(CardType.PLAYER);
            game.getPlayerCards().add(playerCard);
        }

        CardEntity dealerCard = drawCardFromDeck(game);
        dealerCard.setCardType(CardType.DEALER);
        game.getDealerCards().add(dealerCard);

        game.setState(getState(game));
        user.setBalance(user.getBalance().subtract(betAmount));

        userRepository.save(user);
        blackJackGameRepository.save(game);

        return game;
    }

    public BlackJackGameEntity hit(BlackJackGameEntity game) {
        CardEntity drawnCard = drawCardFromDeck(game);
        drawnCard.setCardType(CardType.PLAYER);
        game.getPlayerCards().add(drawnCard);

        game.setState(getState(game));

        return blackJackGameRepository.save(game);
    }

    private void initializeDeck(BlackJackGameEntity game) {
        for (Suit suit : Suit.values()) {
            for (Rank rank : Rank.values()) {
                CardEntity card = new CardEntity();
                card.setGame(game);
                card.setSuit(suit);
                card.setRank(rank);
                card.setCardType(CardType.DECK);
                game.getDeck().add(card);
            }
        }

        java.util.Collections.shuffle(game.getDeck(), random);
    }

    private CardEntity drawCardFromDeck(BlackJackGameEntity game) {
        if (game.getDeck().isEmpty()) {
            throw new IllegalStateException("Deck is empty");
        }

        return game.getDeck().removeFirst();
    }

    private BlackJackState getState(BlackJackGameEntity game) {
        int playerHandValue = calculateHandValue(game.getPlayerCards());

        if (playerHandValue == 21) {
            return BlackJackState.PLAYER_WON;
        } else if (playerHandValue > 21) {
            return BlackJackState.PLAYER_LOST;
        }

        return BlackJackState.IN_PROGRESS;
    }

    private int calculateHandValue(List<CardEntity> hand) {
        int sum = 0;
        int aceCount = 0;
        for (CardEntity card : hand) {
            sum += card.getRank().getValue();
            if (card.getRank() == Rank.ACE) {
                aceCount++;
            }
        }

        while (sum > 21 && aceCount > 0) {
            sum -= 10;
            aceCount--;
        }

        return sum;
    }
}
